<cfscript>
/**
 * Component of merging algorithms with some added caching.
 * 
 * @output false
 */
component
{
	/**
	 * Combines two arrays of sorted values using a predicate. This is an example
	 * where caching probably doesn't benefit us much unless comparator() is extremely slow.
	 * The work to create the cache element as we go is slower then just running comparator().
	 *
	 * @a          Array 1
	 * @b          Array 2
	 * @comparator Function to sort with. Must take two arguments (a,b) to compare.
	 * 
	 * @output false
	 */
	public array function mergeArrays(required array a, required array b, required any comparator)
	{
		var result = [];
		var cache = {};
		var i=1;
		var j=1;
		
		while( i lte arrayLen(a) )
		{
			var comparison = 1;
			
			if( j lte arrayLen(b) )
			{
				if( isSimpleValue(a[i]) and isSimpleValue(b[j]) )
				{
					var key = a[i] & b[j];
				
					if( not structKeyExists(cache,key) )
					{
						cache[key] = comparator(a[i],b[j]);
					}
				
					comparison = cache[key];
				}
				else
				{
					comparison = comparator(a[i],b[j]);
				}
			}
			
			if( j gt arrayLen(b) or comparison <= 0 )
			{
				arrayAppend(result,a[i]);
				++i;
			}
			else
			{
				arrayAppend(result,b[j]);
				++j;
			}
		}
		
		for( ; j lte arrayLen(b); ++j )
			arrayAppend(result,b[j]);
			
		return result;
	}
}
</cfscript>