<cfcomponent output="false">
	
	<cffunction name="init" access="public" returntype="any" output="false">
		<cfargument name="datasource" type="string" required="true">
		<cfargument name="worksheetDirectory" type="string" required="true">

		<cfset setDatasource(arguments.datasource)>
		<cfset setWorksheetDirectory(arguments.worksheetDirectory)>

		<cfreturn this>
	</cffunction>

	<!---+
		Gets a worksheet by name for an event.
	--->
	<cffunction name="getWorksheet" access="public" returntype="any" output="false">
		<cfargument name="eventId" type="string" required="true">
		<cfargument name="name" type="string" required="true">
				
		<cfreturn createObject("component","Worksheet")
			.init(getDatasource(),eventId,getWorksheetDirectory(),name)>
	</cffunction>
	
	<!---+
		Gets a query with the worksheet totals and performs deduplication on the
		topics. Two topics with the same speaker and topic name are considered dups.
	--->
	<cffunction name="getWorksheetTotals" access="public" returntype="query" output="false">
		<cfargument name="eventId" type="string" required="true">
		
		<cfset var files = "">
		<cfset var worksheet = "">
		<cfset var records = "">
		<cfset var key = "">
		<cfset var totals = structNew()>
		
		<cfdirectory 
			directory="#getWorksheetDirectory()#/#eventId#"
			action="list" 
			name="files"
			filter="*.xml"
			listinfo="name">
		
		<!--- 
			Load an empty worksheet last to ensure that the totals start at 0 
			and that we have something to join against.
		--->
		<cfset queryAddRow(files)>
		<cfset querySetCell(files,"name","")>
		
		<cfloop query="files">
			<cfset worksheet = getWorksheet(eventId,reReplaceNoCase(files.name,"\.xml$","")).load()>
			<cfset records = worksheet.getRecords()>
			<cfloop query="records">
				<!--- Generate a key so that duplicate topics are treated the same --->
				<cfset key = records.topicName & chr(1) & 
					records.speakerFirstName & chr(1) & 
					records.speakerLastName>
				
				<cfif not structKeyExists(totals,key)>
					<cfset totals[key] = {
						worksheets = structNew(),
						topicIds = structNew(),
						topicId = records.topicId,
						alternateTracks = structNew()
					}>
				</cfif>
				
				<cfif records.vote>
					<cfset totals[key].worksheets[worksheet.getName()] = true>
				</cfif>
				
				<cfset totals[key].topicIds[records.topicId] = true>
				
				<!--- Keep track of topic changes --->
				<cfif records.trackId neq records.originalTrackId>
					<cfif not structKeyExists(totals[key].alternateTracks,records.trackId)>
						<cfset totals[key].alternateTracks[records.trackId] = {
							count = 0,
							worksheets = structNew()
						}>
					</cfif>
					
					<cfif not structKeyExists(totals[key].alternateTracks[records.trackId],worksheet.getName())>
						<cfset totals[key].alternateTracks[records.trackId].count = 1>
						<cfset totals[key].alternateTracks[records.trackId].worksheets[worksheet.getName()] = true>
					</cfif>
				</cfif>
			</cfloop>
		</cfloop>
		
		<cfset result = queryNew("topicId,topicIds,totalVotes,alternateTracks,worksheets","integer,varchar,integer,object,varchar")>
		<cfloop collection="#totals#" item="key">
			<cfset queryAddRow(result)>
			<cfset querySetCell(result,"topicId",totals[key].topicId)>
			<cfset querySetCell(result,"topicIds",structKeyList(totals[key].topicIds))>
			<cfset querySetCell(result,"totalVotes",structCount(totals[key].worksheets))>
			<cfset querySetCell(result,"alternateTracks",totals[key].alternateTracks)>
			<cfset querySetCell(result,"worksheets",structKeyList(totals[key].worksheets))>
		</cfloop>
		
		<cfquery name="result" dbtype="query">
			select *
			from result, records
			where
				records.topicId = result.topicId
			order by
				result.totalVotes desc,
				records.topicName asc
		</cfquery>
		
		<cfloop query="result">
			<cfif structCount(result.alternateTracks) eq 1>
				<cfset querySetCell(result,"trackId",structKeyList(result.alternateTracks),result.currentRow)>
			</cfif>
		</cfloop>
				
		<cfreturn result>
	</cffunction>
	
	<cffunction name="setLoginService" access="public" returntype="void" output="false">
		<cfargument name="value" type="any" required="true">

		<cfset variables.loginService = arguments.value>
	</cffunction>
	
	<cffunction name="getLoginService" access="public" returntype="any" output="false">

		<cfreturn variables.loginService>
	</cffunction>
	
	<cffunction name="setWorksheetDirectory" access="public" returntype="void" output="false">
		<cfargument name="value" type="string" required="true">

		<cfset variables.worksheetDirectory = expandPath(arguments.value)>
	</cffunction>
	
	<cffunction name="getWorksheetDirectory" access="public" returntype="any" output="false">

		<cfreturn variables.worksheetDirectory>
	</cffunction>
	
	<cffunction name="setDatasource" access="public" returntype="void" output="false">
		<cfargument name="value" type="string" required="true">

		<cfset variables.datasource = arguments.value>
	</cffunction>
	
	<cffunction name="getDatasource" access="public" returntype="any" output="false">

		<cfreturn variables.datasource>
	</cffunction>

</cfcomponent>